package com.emato.ich.fragment;

import android.os.Bundle;
import android.text.Editable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import com.emato.ich.MainActivity;
import com.emato.ich.R;
import com.emato.ich.api.ICSPClient;
import com.emato.ich.api.ICSPResponseCodeEnum;
import com.emato.ich.databinding.FragmentTakeCodeBinding;
import com.emato.ich.device.DeviceControl;
import com.emato.ich.entity.vo.ResponseData;
import com.emato.ich.entity.vo.TakeParcelVo;
import com.emato.ich.local.LocalStorage;
import com.emato.ich.utils.BaseUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.android.material.snackbar.Snackbar;

import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.util.Arrays;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Response;

/**
 * 取件码输入页面
 */
public class TakeCodeFragment extends Fragment {

    private FragmentTakeCodeBinding binding;
    private static final String TAG = TakeCodeFragment.class.getName();

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {

        binding = FragmentTakeCodeBinding.inflate(inflater, container, false);

        return binding.getRoot();
    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        // 选中输入框
//        binding.editTextTextPersonName6.requestFocus();

        // 输入逻辑
        binding.editTextTextPersonName6.setOnFocusChangeListener((view1, hasFocus) -> {
            if (hasFocus) {
                binding.takeInputButton0.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"0");
                });
                binding.takeInputButton1.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"1");
                });
                binding.takeInputButton2.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"2");
                });
                binding.takeInputButton3.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"3");
                });
                binding.takeInputButton4.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"4");
                });
                binding.takeInputButton5.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"5");
                });
                binding.takeInputButton6.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"6");
                });
                binding.takeInputButton7.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"7");
                });
                binding.takeInputButton8.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"8");
                });
                binding.takeInputButton9.setOnClickListener(v -> {
                    int start = binding.editTextTextPersonName6.getSelectionStart();
                    binding.editTextTextPersonName6.getText().insert(start,"9");
                });
                binding.takeInputButtonCancel.setOnClickListener(v -> {
                    Editable text = binding.editTextTextPersonName6.getText();
                   if (text.length() > 0) {
                       text.delete(text.length()-1, text.length());
                       binding.editTextTextPersonName6.setText(text, TextView.BufferType.EDITABLE);
                       binding.editTextTextPersonName6.setSelection(binding.editTextTextPersonName6.getText().length());
                   }
                });
            }
        });

        binding.takeInputButtonConfirm.setOnClickListener(view1 -> {
            // TODO 取件逻辑
            String takeParcelCode = binding.editTextTextPersonName6.getText().toString();
            TakeParcelVo takeParcelVo = new TakeParcelVo();
            takeParcelVo.setClientId(BaseUtils.getClientId());
            takeParcelVo.setPickUp(takeParcelCode);
            takeParcelVo.setReopen("0");
            ICSPClient.takeParcel("", takeParcelVo, new Callback() {
                @Override
                public void onFailure(@NotNull Call call, @NotNull IOException e) {
                    // TODO 取件接口请求失败逻辑处理
                    Snackbar.make(view, "取件失败！网络异常！", Snackbar.LENGTH_LONG).show();
                    Log.e(TAG, "onFailure: 取件请求失败!", e);
                }

                @Override
                public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
                    // TODO 返回锁板和锁id, 开柜门
                    getActivity().runOnUiThread(() -> {
                        String parseResponse = ICSPClient.isSuccessfulAndParseResponse(response);
                        try {
                            ObjectMapper objectMapper = new ObjectMapper();
                            ResponseData responseData = null;
                            try {
                                responseData = objectMapper.readValue(parseResponse, new TypeReference<ResponseData>() {
                                });
                                ICSPResponseCodeEnum.responseHint(view1, responseData);
                            } catch (JsonProcessingException e) {
                                Log.e(TAG, "onResponse: 解析取件开门响应信息失败! ", e);
                            } catch (RuntimeException e) {
                                Log.e(TAG, "onResponse: 未知错误! ", e);
                                Snackbar.make(view1, "未知错误！", Snackbar.LENGTH_LONG).show();
                            }
                            if (null != responseData && responseData.getCode().equals(ICSPResponseCodeEnum.OK.getCode())) {
                                // 开门
                                Log.i(TAG, "onResponse: 取件成功! ");
                                Snackbar.make(view1, "取件成功！", Snackbar.LENGTH_LONG).show();
                                Bundle bundle = new Bundle();
                                bundle.putString("takeParcelCode", takeParcelCode);
                                ((MainActivity) getActivity()).getBundleMap().put(TakeCodeFragment.class.getName(), bundle);
                                NavHostFragment.findNavController(TakeCodeFragment.this).navigate(R.id.action_takeCodeFragment_to_takeSuccessFragment);
                            }
                        } catch (RuntimeException e) {
                            Log.e(TAG, "onResponse: 取件失败! ", e);
                            Snackbar.make(view1, "取件失败！", Snackbar.LENGTH_LONG).show();
                        }
                    });
                }
            });

        });
        binding.returnBtn.setOnClickListener(view1 -> NavHostFragment.findNavController(TakeCodeFragment.this)
        .navigate(R.id.action_takeCodeFragment_to_takeFragment));
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        binding = null;
    }
}
