package com.emato.ich.fragment;

import android.os.Bundle;
import android.os.CountDownTimer;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import com.emato.ich.MainActivity;
import com.emato.ich.R;
import com.emato.ich.api.ICSPClient;
import com.emato.ich.api.ICSPResponseCodeEnum;
import com.emato.ich.api.SystemConfigConstant;
import com.emato.ich.databinding.FragmentTakeSuccessBinding;
import com.emato.ich.entity.vo.ResponseData;
import com.emato.ich.entity.vo.TakeParcelVo;
import com.emato.ich.local.LocalStorage;
import com.emato.ich.utils.BaseUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.android.material.snackbar.Snackbar;

import org.jetbrains.annotations.NotNull;

import java.io.IOException;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Response;

public class TakeSuccessFragment extends Fragment {

    private static final String TAG = TakeSuccessFragment.class.getName();
    private FragmentTakeSuccessBinding binding;

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {

        binding = FragmentTakeSuccessBinding.inflate(inflater, container, false);

        return binding.getRoot();

    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {

        binding.continueTakeBtn.setOnClickListener(view1 -> NavHostFragment.findNavController(TakeSuccessFragment.this)
        .navigate(R.id.action_takeSuccessFragment_to_takeCodeFragment));

        binding.returnMainBtn.setOnClickListener(view1 -> NavHostFragment.findNavController(TakeSuccessFragment.this)
            .navigate(R.id.action_takeSuccessFragment_to_mainFragment));

        try {
            MainActivity activity = (MainActivity) getActivity();
            String s = activity.getConfigMap().get(SystemConfigConstant.cabinet_take_object_reopen_time);
            // 倒计时
            int i = Integer.parseInt(s);
            CountDownTimer timer = new CountDownTimer(i * 1000, 1000) {
                @Override
                public void onTick(long millisUntilFinished) {
                    binding.openCabinetCountDownHint.setText(String.format("倒计时%s秒，只有一次机会，超时后不可开门！", (millisUntilFinished / 1000)));
                }

                @Override
                public void onFinish() {
                    binding.openCabinetCountDownHint.setText("已超时，不可开门！");
                    binding.reOpenCabinetBtn.setEnabled(false);
                }
            };
            timer.start();
        } catch (RuntimeException e) {
            Log.e(TAG, "onViewCreated: 倒计时未知异常! ", e);
        }

        binding.reOpenCabinetBtn.setOnClickListener(view1 -> {

            // TODO 取件逻辑
            String takeParcelCode = null;
            MainActivity activity = null;
            try {
                activity = (MainActivity) getActivity();
                Bundle bundle = activity.getBundleMap().get(TakeCodeFragment.class.getName());
                takeParcelCode = bundle.getString("takeParcelCode");
            } catch (RuntimeException e) {
                Log.e(TAG, "onViewCreated: 获取TakeCodeFragment传值TakeSuccessFragment失败! ", e);
            }
            TakeParcelVo takeParcelVo = new TakeParcelVo();
            takeParcelVo.setClientId(BaseUtils.getClientId());
            takeParcelVo.setPickUp(takeParcelCode);
            takeParcelVo.setReopen("1");
            MainActivity finalActivity = activity;
            ICSPClient.takeParcel(LocalStorage.getInstance().getSession().getToken(), takeParcelVo, new Callback() {
                @Override
                public void onFailure(@NotNull Call call, @NotNull IOException e) {
                    // TODO 取件接口请求失败逻辑处理
                    Snackbar.make(view, "取件失败！网络异常！", Snackbar.LENGTH_LONG).show();
                    Log.e(TAG, "onFailure: 取件请求失败!", e);
                }

                @Override
                public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
                    // TODO 返回锁板和锁id, 开柜门
                    getActivity().runOnUiThread(() -> {
                        String parseResponse = ICSPClient.isSuccessfulAndParseResponse(response);
                        try {
                            ObjectMapper objectMapper = new ObjectMapper();
                            ResponseData responseData = null;
                            try {
                                responseData = objectMapper.readValue(parseResponse, new TypeReference<ResponseData>() {
                                });
                                ICSPResponseCodeEnum.responseHint(view1, responseData);
                            } catch (JsonProcessingException e) {
                                Log.e(TAG, "onResponse: 解析取件开门响应信息失败! ", e);
                            } catch (RuntimeException e) {
                                Log.e(TAG, "onResponse: 未知错误! ", e);
                                Snackbar.make(view1, "未知错误！", Snackbar.LENGTH_LONG).show();
                            }
                            if (null != responseData && responseData.getCode().equals(ICSPResponseCodeEnum.OK.getCode())) {
                                // 开门
                                Log.i(TAG, "onResponse: 取件成功! ");
                                Snackbar.make(view1, "取件成功！", Snackbar.LENGTH_LONG).show();
                                finalActivity.getBundleMap().remove(TakeCodeFragment.class.getName());
                            } else {
                                ICSPResponseCodeEnum.responseHint(view1, responseData);
                            }
                        } catch (RuntimeException e) {
                            Log.e(TAG, "onResponse: 取件失败! ", e);
                            Snackbar.make(view1, "取件失败！", Snackbar.LENGTH_LONG).show();
                        }
                    });
                }
            });
        });
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        binding = null;
    }
}
